#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""
import numpy as np 

def initialCond(D, nx, ny, flag_Outlet):
    Uxo = np.zeros((ny, nx+1))
    Uyo = np.zeros((ny+1, nx))
    Po = np.zeros((ny, nx))
        
        
    for i in range (0, ny):
        for j in range (0, nx+1):
            if(i==0):
                if((D[0][i, j]==0 and D[0][i+1, j]==0) or (D[0][i, j]==1 and D[0][i+1, j]==1)):
                    Uxo[i, j] = D[1][i+1, j]
            else:
                if((D[0][i, j]==0 and D[0][i+1, j]==0) or (D[0][i, j]==1 and D[0][i+1, j]==1)):
                    Uxo[i, j] = D[1][i, j]

    
                
    for i in range (0, ny+1):
        for j in range (0, nx):
            if(j==0):
                if((D[0][i, j]==0 and D[0][i, j+1]==0) or (D[0][i, j]==1 and D[0][i, j+1]==1)):
                    Uyo[i, j] = D[2][i, j+1]
            else:
                if((D[0][i, j]==0 and D[0][i, j+1]==0) or (D[0][i, j]==1 and D[0][i, j+1]==1)):
                    Uyo[i, j] = D[2][i, j]  
    if(flag_Outlet==1):
        for i in range (0, ny):
            for j in range (0, nx):
                if(D[0][i, j]==-1):
                    Po[i, j] = D[3][i, j]
                if(D[0][i+1, j]==-1):
                    Po[i, j] = D[3][i+1, j]
                if(D[0][i, j+1]==-1):
                    Po[i, j] = D[3][i, j+1]
                if(D[0][i+1, j+1]==-1):
                    Po[i, j] = D[3][i+1, j+1]
                
    return Uxo, Uyo, Po


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""
#%%
import numpy as np 
import matplotlib as mpl
from matplotlib import cm
from matplotlib import pyplot as plt 
from scipy import linalg 
import sys, os
mpl.rcParams.update({'font.size': 15})
caseFolder = input("\nEnter address of testCase ...\nExample:  /home/user/folder/Case_1\n") 
try:
    os.mkdir(caseFolder)
except: 
    print("\nFolder already there!!\n")
    flag_EmptyFolder = input("\nEmpty Folder ?? [y/n]\n")
    if(flag_EmptyFolder=='y'):
        filelist = [ f for f in os.listdir(caseFolder) ]
        for f in filelist:
            os.remove(os.path.join(caseFolder, f))
    else:
        sys.exit()

package_dir = os.path.dirname(os.path.realpath(__file__))
print(package_dir)
from RUN_package.boundaryDomain import domain, obstacle, obstacleMath
from RUN_package.initialConditions import initialCond
from RUN_package.faceInterpolation import U_face, V_face
from RUN_package.UV_coeff import U_coeffUpwind, V_coeffUpwind
from RUN_package.P_coeff import P_coeffSIMPLE
from RUN_package.correction import correctionSIMPLE
from RUN_package.postProcessing import postProcessor
D, \
xMin, xMax, yMin, yMax, \
dx, dy, nx, ny, \
south_Boundary, north_Boundary, west_Boundary, east_Boundary, \
flag_Outlet =\
domain(caseFolder)
flag_obstacleGeometry = int(input("\n1. Use .txt file as Address \n2. Add elliptical obstacle\n3. No Obstacle Geometry\n"))
if(flag_obstacleGeometry==1):
    D, Points, Points_x, Points_y = obstacle(D, xMin, xMax, yMin, yMax, nx, ny, dx, dy)   
elif(flag_obstacleGeometry==2):
    D, Points, Points_x, Points_y = obstacleMath(D, xMin, xMax, yMin, yMax, nx, ny, dx, dy)
Lx = xMax-xMin
Ly = yMax-yMin

# Grid - Plot 
xg = np.linspace(xMin, xMax, num=nx+2)
yg = np.linspace(yMax, yMin, num=ny+2)
 
Xg, Yg = np.meshgrid(xg, yg)
fig, ax = plt.subplots(figsize =(Lx*6*1.4, Ly*6))
plt.xlabel('x [m]')
plt.ylabel('y [m]')
cmap = plt.get_cmap('tab10')
ylorbr = cm.get_cmap('tab10', 4)
norm = mpl.colors.Normalize(vmin=-1.5, vmax=2.5)
col = plt.pcolormesh(xg, yg, D[0], cmap = ylorbr, norm=norm)
cbar = plt.colorbar(col)
cbar.set_label('Grid\n-1.0 - Outlet    0.0 - Wall    1.0 - Inlet    2.0 - Wall')
plt.axis('scaled')

# show plot
plt.show()

flag_continue = input("\nContinue or Abort ?? [y/n]\n")
if(flag_continue=='n'):
    sys.exit()


urf_UV = float(input("\nEnter Under-relaxation factor for UV\n"))
urf_P = float(input("\nEnter Under-relaxation factor for P\n"))
rho = float(input("\nEnter Density of fluid [kg/m^3]\n"))
visc = float(input("\nEnter Viscosity of Fluid [Pa*s]\n"))
max_iter = int(input("\nEnter Max Iteration for this SIM\n"))
MaxRes_Mass = float(input("\nEnter Min Mass Imbalance Residual for this SIM\n"))
    
Uxo, Uyo, Po = initialCond(D, nx, ny, flag_Outlet)
Ux_values=[]
Ux_values.append(Uxo)
Uy_values=[]
Uy_values.append(Uyo)
P_values=[]
P_values.append(Po)
Res_U_values=[]
Res_V_values=[]
Res_Mass_values=[]
logRes_Mass=0.0
n=0
iter_ = np.linspace(0, max_iter, max_iter)
temp = np.linspace(0, max_iter, max_iter)
plt.ion()
fig = plt.figure()
ax = fig.add_subplot(111)
line1, = ax.plot(iter_, temp, color = 'red', label = 'Ux-Mom Residual')
line2, = ax.plot(iter_, temp, color = 'blue', label = 'Uy-Mom Residual')
line3, = ax.plot(iter_, temp, color = 'green', label = 'Mass Residual')
ax.legend()

while n<max_iter and logRes_Mass>MaxRes_Mass:
    Uue, Uuw, Vun, Vus = U_face(Uxo, Uyo, D, nx, ny)
    Uve, Uvw, Vvn, Vvs = V_face(Uxo, Uyo, D, nx, ny)
    
    
    
    M_Au, M_Bup, Aup, Aue, Auw, Aun, Aus, Bup = \
        U_coeffUpwind(Uxo, Uyo, Po, Uue, Uuw, Vun, Vus, D, rho, visc, urf_UV, nx, ny, dx, dy)
    M_Av, M_Bvp, Avp, Ave, Avw, Avn, Avs, Bvp = \
        V_coeffUpwind(Uxo, Uyo, Po, Uve, Uvw, Vvn, Vvs, D, rho, visc, urf_UV, nx, ny, dx, dy)
    
    
    
    # Momentum Predictor 
    M_Uxs = linalg.solve(M_Au, M_Bup)
    M_Uys = linalg.solve(M_Av, M_Bvp)
    Uxs = np.reshape(M_Uxs, (ny, nx+1))
    Uys = np.reshape(M_Uys, (ny+1, nx))

    
    M_Ap, M_Bpp, App, Ape, Apw, Apn, Aps, Bpp = P_coeffSIMPLE(Aup, Avp, Uxs, Uys, D, nx, ny, dx, dy)
    if(flag_Outlet==0):
        M_Ap[int(ny*nx/4), :] = 0
        M_Ap[int(ny*nx/4), int(ny*nx/4)] = 1
        M_Bpp[int(ny*nx/4)] = 0
    
    # Pressure Correction values
    M_Pc = linalg.solve(M_Ap, M_Bpp)
    
    Pc = np.reshape(M_Pc, (ny, nx))
    
    # Residuals 
    Res_U = M_Au.dot(Uxo.flatten())-M_Bup
    Res_V = M_Av.dot(Uyo.flatten())-M_Bvp
    logRes_U = np.log(np.sum(np.abs(Res_U)))
    logRes_V = np.log(np.sum(np.abs(Res_V)))
    logRes_Mass = np.log(np.sum(np.abs(Bpp)))
    
    # Momentum & Pressure Correction step
    Ux, Uy, P = correctionSIMPLE(Uxs, Uys, Po, Pc, D, Aup, Avp, urf_UV, urf_P, nx, ny, dx, dy)
    
    # Writing Results 
    Ux_values.append(Ux.copy())
    Uy_values.append(Uy.copy())
    P_values.append(P.copy())
    Res_U_values.append(logRes_U)
    Res_V_values.append(logRes_V)
    Res_Mass_values.append(logRes_Mass)
    
    # Prepping for next step 
    Uxo = Ux.copy()
    Uyo = Uy.copy()
    Po = P.copy()
    iter_ = np.linspace(0, n+1, n+1)
    ax.set_xlim(-0.5, n+2)
    max_Res=0
    min_Res=0
    for k in range (0, n+1):
        if(max_Res<Res_U_values[k]):
            max_Res=Res_U_values[k]
        if(max_Res<Res_V_values[k]):
            max_Res=Res_U_values[k]
        if(max_Res<Res_Mass_values[k]):
            max_Res=Res_Mass_values[k]
        if(min_Res>Res_U_values[k]):
            min_Res=Res_U_values[k]
        if(min_Res>Res_V_values[k]):
            min_Res=Res_U_values[k]
        if(min_Res>Res_Mass_values[k]):
            min_Res=Res_Mass_values[k]
    
    ax.set_ylim(min_Res-1, max_Res+1)
    line1.set_ydata(Res_U_values)
    line1.set_xdata(iter_)
    line2.set_ydata(Res_V_values)
    line2.set_xdata(iter_)
    line3.set_ydata(Res_Mass_values)
    line3.set_xdata(iter_)

    fig.canvas.draw()
    fig.canvas.flush_events()
    n+=1


if(n<max_iter):
    max_iter=n


max_iter
uxq_values = []
uyq_values = []
for n in range (0, max_iter):
    ux1 = Ux_values[n].copy()
    uxq = np.zeros((2*ny+1, 2*nx+1))
    for i in range (0, ny):
        for j in range (0, nx+1):
            uxq[2*i+1, 2*j] = ux1[i, j]
    for j in range (0, nx+1):
        uxq[0, 2*j] = D[1][0, j]
        uxq[-1, 2*j] = D[1][-1, j]
    for i in range (0, ny-1):
        for j in range (0, nx+1):
            uxq[2*i+2, 2*j] = (uxq[2*i+1, 2*j] + uxq[2*i+3, 2*j])/2
    for i in range (0, 2*ny):
        for j in range (0, nx):
            uxq[i, 2*j+1] = (uxq[i, 2*j] + uxq[i, 2*j+2])/2

    
    uy1 = Uy_values[n].copy()
    uyq = np.zeros((2*ny+1, 2*nx+1))

    
    for i in range (0, ny+1):
        for j in range (0, nx):
            uyq[2*i, 2*j+1] = uy1[i, j]
    for i in range (0, ny+1):
        uyq[2*i, 0] = D[2][i, 0]
        uyq[2*i, -1] = D[2][i, -1]
    for i in range (0, ny+1):
        for j in range (0, nx-1):
            uyq[2*i, 2*j+2] = (uyq[2*i, 2*j+1] + uyq[2*i, 2*j+3])/2
    for i in range (0, ny):
        for j in range (0, 2*nx):
            uyq[2*i+1, j] = (uyq[2*i, j] + uyq[2*i+2, j])/2

    uxq_values.append(uxq)
    uyq_values.append(uyq)


#%% 
flag_Validation = input("Validation check for Lid-driven Cavity Re-[100, 1000, 5000] ?? [y/n]")
if(flag_Validation=='y'):
    mpl.rcParams.update({'font.size': 10})
    ghia_u100 = np.array([1, 0.84123, 0.78871, 0.73722, 0.68717, 0.23151, 0.00332, -0.13641, -0.20581, -0.2109, -0.15662, -0.1015, -0.06434, -0.04775, -0.04192, -0.03717, 0])
    ghia_u1000 = np.array([1, 0.65928, 0.57492, 0.51117, 0.46604, 0.33304, 0.18719, 0.05702, -0.0608, -0.10648, -0.27805, -0.38289, -0.2973, -0.2222, -0.20196, -0.18109, 0])
    ghia_u5000 = np.array([1, 0.48223, 0.4612, 0.45992, 0.46036, 0.33556, 0.20087, 0.08183, -0.03039, -0.07404, -0.22855, -0.3305, -0.40435, -0.43643, -0.42901, -0.41165, 0])
    ghia_y = np.array([1, 0.9766, 0.9688, 0.9609, 0.9531, 0.8516, 0.7344, 0.6172, 0.5, 0.4531, 0.2813, 0.1719, 0.1016, 0.0703, 0.0625, 0.0547, 0])
    yq = np.linspace(yMax, yMin, num=2*ny+1)  
    Re = rho*D[1][0, int(nx/2)]*Lx/visc
    if(Re==100):
        plt.scatter(ghia_u100, ghia_y, c='green', label='Ghia etal.')
        plt.title('Reynolds No. 100 Mesh -[{}X{}]'.format(nx, ny))
    if(Re==1000):
        plt.scatter(ghia_u1000, ghia_y, c='green', label='Ghia etal.')
        plt.title('Reynolds No. 1000 Mesh -[{}X{}]'.format(nx, ny))
    if(Re==5000):
        plt.scatter(ghia_u5000, ghia_y, c='green', label='Ghia etal.')
        plt.title('Reynolds No. 5000 Mesh -[{}X{}]'.format(nx, ny))
    plt.plot(uxq_values[max_iter-1][:, nx], yq, label ='Centreline X-Velocity')
    plt.legend()
    plt.show()
flag_Validation = input("Validation check for Fully developed flow ?? [y/n]")
if(flag_Validation=='y'):
    mpl.rcParams.update({'font.size': 10})
    Re = rho*D[1][int(ny/2), 0]*Ly/visc
    yq = np.linspace(yMax, yMin, num=2*ny+1)  
    plt.plot(uxq_values[max_iter-1][:, int(8*(2*nx+1)/10)], yq, label ='Centreline X-Velocity - at X={}'.format(xMin+int(8*(2*nx+1)/10)*dx/2))
    # plt.set_xlabel('Y')
    plt.title('Fully Developed Flow Re = {}'.format(Re))
    plt.legend()
    plt.show()
    xq = np.linspace(xMin, xMax, num=2*nx+1)  
    plt.plot(uxq_values[max_iter-1][ny, :], xq, label ='Centreline X-Velocity - at Y={}'.format(yMax-ny*dy/2))
    # plt.set_xlabel('X')    
    plt.title('Fully Developed Flow Re = {}'.format(Re))
    plt.legend()
    plt.show()

#%%

postProcessor(P_values, uxq_values, uyq_values, \
                  xMin, xMax, yMin, yMax, nx, ny, dx, dy, Points_x, Points_y, \
                  caseFolder, Res_U_values, Res_V_values, Res_Mass_values, \
                  max_iter, min_Res, max_Res)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""
import numpy as np
import matplotlib
from matplotlib import pyplot as plt 
import matplotlib.animation as animation
matplotlib.rcParams.update({'font.size': 15})
def postProcessor(P_values, uxq_values, uyq_values, xMin, xMax, yMin, yMax, nx, ny, dx, dy, Points_x, Points_y, caseFolder, Res_U_values, Res_V_values, Res_Mass_values, max_iter, min_Res, max_Res):
    ('plasma')
    flag_generateResults = int(input("\n1. Generate Results\n2. Abort\n"))
    flag_newMap = 'y'
    flag_newMapOK = 2
    if(flag_generateResults==1):
        flag_generateResidualsPlot = input("Generate Residuals plot ?? [y/n]")
        if(flag_generateResidualsPlot=='y'):
            fig, ax = fig, ax = plt.subplots(figsize =(10, 5))
            iter_ = np.linspace(0, max_iter, max_iter)
            ax.plot(iter_, Res_U_values, color = 'red', label = 'Ux-Mom Residual')
            ax.plot(iter_, Res_V_values, color = 'blue', label = 'Uy-Mom Residual')
            ax.plot(iter_, Res_Mass_values, color = 'green', label = 'Mass Residual')
            ax.set_ylim(min_Res, max_Res)
            ax.set_xlim(0, max_iter)
            ax.set_xlabel('Iterations')
            ax.legend()
            plt.savefig(caseFolder+"/Residuals.jpg")

        flag_generateCSV = int(input("\n[1]- Generate CSV files of Residuals \n[2]- Generate CSV files of U, V and P\n[3]- No CSV Results\n"))
        if(flag_generateCSV==1):
            np.savetxt(caseFolder+"/U_Residual.csv", Res_U_values, delimiter=",")
            np.savetxt(caseFolder+"/V-Residual.csv", Res_V_values, delimiter=",")
            np.savetxt(caseFolder+"/Mass_Residual.csv", Res_Mass_values, delimiter=",")
        elif(flag_generateCSV==2):
            np.savetxt(caseFolder+"/U.csv", uxq_values[max_iter-1], delimiter=",")
            np.savetxt(caseFolder+"/V.csv", uyq_values[max_iter-1], delimiter=",")
            np.savetxt(caseFolder+"/P.csv", P_values[max_iter], delimiter=",")
        mapCounter = 1
        matplotlib.rcParams.update({'font.size': 35})
        while(flag_newMap=='y'):
            print("Generating Map-", mapCounter)
            flag_resultColorMap = input("Add ColorMap ?? [y/n]")
            flag_resultGeometry = input("Add Black Obstacle Geometry ?? [y/n]")
            if(flag_resultColorMap=='y'):
                flag_resultColorMapMarker = int(input("\nSelect Variable\n[1]- Pressure\n[2]- Velocity\n[3]- X-Velocity\n[4]- Y-Velocity\n"))
                flag_resultColorMapContourlines = input("\nWith Labelled Contourlines ?? [y/n]\n")
                if(flag_resultColorMapContourlines=='y'):
                    ContourlinesDensity = int(20/int(input("\nDensity of Contourlines ??\n")))
                min_resultColorMap = float(input("Min value of variable"))
                max_resultColorMap = float(input("Max value of variable"))
            flag_resultStreamlines = input("Add Streamlines ?? [y/n]")
            if(flag_resultStreamlines=='y'):
                flag_resultStreamlinesDensity = int(input("Streamlines Density ??"))
            flag_resultVectors = input("Add Vectors ?? [y/n]")
            
            
            xq = np.linspace(xMin, xMax, num=2*nx+1)
            yq = np.linspace(yMax, yMin, num=2*ny+1)
            xqs = np.linspace(xMin, xMax, num=2*nx+1)
            yqs = np.linspace(yMin, yMax, num=2*ny+1)
            xp = np.linspace(xMin+dx/2, xMax-dx/2, num=nx)
            yp = np.linspace(yMax-dy/2, yMin+dy/2, num=ny)
            Xq, Yq = np.meshgrid(xqs, yqs)
            # creating plot
            Lx = xMax-xMin
            Ly = yMax-yMin
            fig, ax = plt.subplots(figsize =(Lx*10, Ly*10))
            if(flag_resultGeometry=='y'):
                ax.fill(Points_x, Points_y, color='black')
            if(flag_resultColorMap=='y'):
                levels = np.linspace(min_resultColorMap, max_resultColorMap, num=20)
                if(flag_resultColorMapMarker==1):
                    contf = ax.contourf(xp, yp, P_values[max_iter-1], levels=levels, cmap=plt.cm.plasma)
                    if(flag_resultColorMapContourlines=='y'):
                        cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                        # ax.clabel(cont, colors='w', fontsize=14)
                    cbar = fig.colorbar(contf)
                    cbar.set_label('P')
                elif(flag_resultColorMapMarker==2):
                    contf = ax.contourf(xq, yq, np.sqrt(uxq_values[max_iter-1]**2+uyq_values[max_iter-1]**2), levels=levels, cmap=plt.cm.plasma)
                    if(flag_resultColorMapContourlines=='y'):
                        cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                        # ax.clabel(cont, colors='w', fontsize=14)
                    cbar = fig.colorbar(contf)
                    cbar.set_label('Vel')
                elif(flag_resultColorMapMarker==3):
                    contf = ax.contourf(xq, yq, uxq_values[max_iter-1], levels=levels, cmap=plt.cm.plasma)
                    if(flag_resultColorMapContourlines=='y'):
                        cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                        # ax.clabel(cont, colors='w', fontsize=14)
                    cbar = fig.colorbar(contf)
                    cbar.set_label('Vel_x')
                elif(flag_resultColorMapMarker==4):
                    contf = ax.contourf(xq, yq, uyq_values[max_iter-1], levels=levels, cmap=plt.cm.plasma)
                    if(flag_resultColorMapContourlines=='y'):
                        cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                        # ax.clabel(cont, colors='w', fontsize=14)
                    cbar = fig.colorbar(contf)
                    cbar.set_label('Vel_y')
            if(flag_resultStreamlines=='y'):
                ax.streamplot(Xq, Yq, np.flip(uxq_values[max_iter-1], 0) ,np.flip(uyq_values[max_iter-1], 0), density=flag_resultStreamlinesDensity, color='black', linewidth=1)
                ax.xaxis.set_ticks([])
                ax.yaxis.set_ticks([])
                ax.axis([xMin, xMax, yMin, yMax])
                ax.set_aspect('equal')
            elif(flag_resultVectors=='y'):
                ax.quiver(Xq, Yq, np.flip(uxq_values[max_iter-1], 0) ,np.flip(uyq_values[max_iter-1], 0))
                ax.xaxis.set_ticks([])
                ax.yaxis.set_ticks([])
                ax.axis([xMin, xMax, yMin, yMax])
                ax.set_aspect('equal')
            
            # show plot
            plt.show()
            flag_newMapOK = int(input("\n1. Continue with Map \n2. Generate again \n"))
            if(flag_newMapOK==2):
                continue
            flag_plotall = int(input("\n0. Plot Last frame only [n] - [jpg]\n1. Plot every n iterations - [GIF]\n"))
            if(flag_plotall!=0):
                xq = np.linspace(xMin, xMax, num=2*nx+1)
                yq = np.linspace(yMin, yMax, num=2*ny+1)
                xp = np.linspace(xMin+dx/2, xMax-dx/2, num=nx)
                yp = np.linspace(yMax-dy/2, yMin+dy/2, num=ny)
                Xq, Yq = np.meshgrid(xq, yq)
                Lx = xMax-xMin
                Ly = yMax-yMin
                fig, ax = plt.subplots(figsize =(Lx*10, Ly*10))
                if(flag_resultColorMap=='y'):
                    levels = np.linspace(min_resultColorMap, max_resultColorMap, num=20)
                    if(flag_resultColorMapMarker==1):
                        contf = ax.contourf(xp, yp, P_values[0], levels=levels, cmap=plt.cm.plasma)
                        cbar = fig.colorbar(contf)
                        cbar.set_label('P')
                    elif(flag_resultColorMapMarker==2):
                        contf = ax.contourf(xq, yq, np.sqrt(uxq_values[0]**2+uyq_values[0]**2), levels=levels, cmap=plt.cm.plasma)
                        cbar = fig.colorbar(contf)
                        cbar.set_label('Vel')
                    elif(flag_resultColorMapMarker==3):
                        contf = ax.contourf(xq, yq, uxq_values[0], levels=levels, cmap=plt.cm.plasma)
                        cbar = fig.colorbar(contf)
                        cbar.set_label('Vel_x')
                    elif(flag_resultColorMapMarker==4):
                        contf = ax.contourf(xq, yq, uyq_values[0], levels=levels, cmap=plt.cm.plasma)
                        cbar = fig.colorbar(contf)
                        cbar.set_label('Vel_y')
                def animate(i):
                    ax.clear() 
                    if(flag_resultGeometry=='y'):
                        ax.fill(Points_x, Points_y, color='black')
                    if(flag_resultColorMap=='y'):
                        levels = np.linspace(min_resultColorMap, max_resultColorMap, num=20)
                        if(flag_resultColorMapMarker==1):
                            contf = ax.contourf(xp, yp, P_values[i], levels=levels, cmap=plt.cm.plasma)
                            if(flag_resultColorMapContourlines=='y'):
                                cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                                # ax.clabel(cont, colors='w', fontsize=14)
                            # cbar = fig.colorbar(contf)
                            # cbar.set_label('P')
                        elif(flag_resultColorMapMarker==2):
                            contf = ax.contourf(xq, yq, np.sqrt(uxq_values[i]**2+uyq_values[i]**2), levels=levels, cmap=plt.cm.plasma)
                            if(flag_resultColorMapContourlines=='y'):
                                cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                                # ax.clabel(cont, colors='w', fontsize=14)
                            # cbar = fig.colorbar(contf)
                            # cbar.set_label('Vel')
                        elif(flag_resultColorMapMarker==3):
                            contf = ax.contourf(xq, yq, uxq_values[i], levels=levels, cmap=plt.cm.plasma)
                            if(flag_resultColorMapContourlines=='y'):
                                cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                                # ax.clabel(cont, colors='w', fontsize=14)
                            # cbar = fig.colorbar(contf)
                            # cbar.set_label('Vel_x')
                        elif(flag_resultColorMapMarker==4):
                            contf = ax.contourf(xq, yq, uyq_values[i], levels=levels, cmap=plt.cm.plasma)
                            if(flag_resultColorMapContourlines=='y'):
                                cont = ax.contour(contf, levels=contf.levels[::ContourlinesDensity], colors='r')
                                # ax.clabel(cont, colors='w', fontsize=14)
                            # cbar = fig.colorbar(contf)
                            # cbar.set_label('Vel_y')
                    if(flag_resultStreamlines=='y'):
                        ax.streamplot(Xq, Yq, np.flip(uxq_values[i], 0) ,np.flip(uyq_values[i], 0), density=flag_resultStreamlinesDensity, color='black', linewidth=1)
                    elif(flag_resultVectors=='y'):
                        ax.quiver(Xq, Yq, np.flip(uxq_values[i], 0) ,np.flip(uyq_values[i], 0))
                    
                    ax.set_title('Map-{} Step-[{}]'.format(mapCounter, i))
                interval = 0.1
                ani = animation.FuncAnimation(fig ,animate, max_iter, interval=interval*1e+3,blit=False)
                writergif = animation.PillowWriter(fps=10)
                ani.save(caseFolder+"/Map-{}.gif".format(mapCounter), writer = writergif)
            if(flag_plotall==0):
                fig.savefig(caseFolder+"/Map-{} Step-[{}].jpg".format(mapCounter, max_iter)) 
            flag_newMap = input("Generate another Map ?? [y/n]")
            mapCounter+=1


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""

import numpy as np 

def correctionSIMPLE(Uxs, Uys, Po, Pc, D, Aup, Avp, urf_UV, urf_P, nx, ny, dx, dy):
    P = np.zeros((ny, nx))
    Ux = Uxs.copy()
    Uy = Uys.copy()
    for i in range (0, ny):
        for j in range (0, nx):
            P[i, j] = Po[i, j] + urf_P*Pc[i, j]
            if(D[0][i, j]==2 or D[0][i+1, j]==2):
                Ux[i, j] = Uxs[i, j] + urf_UV*dy*(Pc[i, j-1]-Pc[i, j])/Aup[i, j]
            if(D[0][i, j]==2 or D[0][i, j+1]==2):
                Uy[i, j] = Uys[i, j] + urf_UV*dx*(Pc[i, j]-Pc[i-1, j])/Avp[i, j]             
    for i in range (0, ny):
        for j in range (0, nx+1):
            if(j==nx and D[0][i, j]==-1 and D[0][i+1, j]==-1):
                Ux[i, j] = Ux[i, j-1]

    return Ux, Uy, P
            


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################
"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""

import numpy as np 

def U_coeffUpwind(Uxo, Uyo, Po, Uue, Uuw, Vun, Vus, D, rho, visc, urf_UV, nx, ny, dx, dy):
    Aup = np.zeros((ny, nx+1))
    Aue = np.zeros((ny, nx+1))
    Auw = np.zeros((ny, nx+1))
    Aun = np.zeros((ny, nx+1))
    Aus = np.zeros((ny, nx+1))
    Bup = np.zeros((ny, nx+1))
    vyx = visc*dy/dx
    vxy = visc*dx/dy
    for i in range (0, ny):
        for j in range (0, nx+1):
            if(j!=0 and j!=nx):
                Aup[i, j] = rho*dy*(max(Uue[i, j], 0) + max(-Uuw[i, j], 0)) +\
                            rho*dx*(max(Vun[i, j], 0) + max(-Vus[i, j], 0)) +\
                            2*(vyx+vxy)
                Aue[i, j] = rho*dy*max(-Uue[i, j], 0) + vyx
                Auw[i, j] = rho*dy*max(Uuw[i, j], 0) + vyx
                Aun[i, j] = rho*dx*max(-Vun[i, j], 0) + vxy
                Aus[i, j] = rho*dx*max(Vus[i, j], 0) + vxy
                Bup[i, j] = dy*(Po[i, j-1]-Po[i, j])
            
            # South Boundary Outlet
            if(D[0][i, j] == 2 and D[0][i+1, j] == -1):
                Aup[i, j] += -vxy - rho*dx*max(Vus[i, j], 0)
                Aus[i, j] = 0
            # North Boundary Outlet
            if(D[0][i, j] == -1 and D[0][i+1, j] == 2):
                Aup[i, j] += -vxy - rho*dx*max(-Vun[i, j], 0)
                Aue[i, j] = 0
            # South Boundary Wall
            if(D[0][i, j] == 2 and D[0][i+1, j] == 0):
                Aup[i, j] += 2*vxy
                Aun[i, j] += vxy/3
                Aus[i, j] = 0
                Bup[i, j] += vxy*8*D[1][i+1, j]/3
            # North Boundary Wall
            if(D[0][i, j] == 0 and D[0][i+1, j] == 2):
                Aup[i, j] += 2*vxy
                Aus[i, j] += vxy/3
                Aun[i, j] = 0
                Bup[i, j] += vxy*8*D[1][i, j]/3
            # South Boundary Inlet
            if(D[0][i, j] == 2 and D[0][i+1, j] == 1):
                Aup[i, j] += 2*vxy + 2*rho*dx*max(Vus[i, j], 0)
                Aun[i, j] += vxy/3 + rho*dx*max(Vus[i, j], 0)/3
                Aus[i, j] = 0
                Bup[i, j] += vxy*8*D[1][i+1, j]/3 + 8*rho*dx*D[1][i+1, j]*max(Vus[i, j], 0)/3
            # North Boundary Inlet
            if(D[0][i, j] == 1 and D[0][i+1, j] == 2):
                Aup[i, j] += 2*vxy + 2*rho*dx*max(-Vun[i, j], 0)
                Aus[i, j] += vxy/3 + rho*dx*max(-Vun[i, j], 0)/3
                Aun[i, j] = 0
                Bup[i, j] += vxy*8*D[1][i, j]/3 + 8*rho*dx*D[1][i, j]*max(-Vun[i, j], 0)/3
            

            # Data Point on Boundary 
            if(D[0][i, j]!=2 and D[0][i+1, j]!=2):
                Aup[i, j] = 1
                Aue[i, j] = 0
                Auw[i, j]  = 0
                Aun[i, j]  = 0
                Aus[i, j]  = 0
                Bup[i, j]  = Uxo[i, j]
            # Outlet Boundary 
            if(D[0][i, j]==-1 and D[0][i+1, j]==-1 and j==nx):
                Aup[i, j] = 1
                Aue[i, j] = 0
                Auw[i, j]  = 1
                Aun[i, j]  = 0
                Aus[i, j]  = 0
                Bup[i, j]  = 0
            if(D[0][i, j]==-1 and D[0][i+1, j]==-1 and j==0):
                Aup[i, j] = 1
                Aue[i, j] = 1
                Auw[i, j]  = 0
                Aun[i, j]  = 0
                Aus[i, j]  = 0
                Bup[i, j]  = 0
            # Under-Relaxation Factor
            if(D[0][i, j]==2 or D[0][i+1, j]==2):
                Aup[i, j] = Aup[i, j]/urf_UV
                Bup[i, j] += (1-urf_UV)*Aup[i, j]*Uxo[i, j]

                
    # Matrix Creation 
    M_Bup = Bup.flatten()
    M_Au = np.zeros(((ny)*(nx+1), (ny)*(nx+1)))
    ite=0
    for i in range (0, ny):
        for j in range (0, nx+1):
            M_Au[ite, ite] = Aup[i, j]
            if ((ite+1)%(nx+1)!=0):
                M_Au[ite, ite+1] = -Aue[i, j]
            if((ite%(nx+1)!=0) and (ite!=0)):
                M_Au[ite, ite-1] = -Auw[i, j]
            if (ite<(ny-1)*(nx+1)):
                M_Au[ite, ite+nx+1] = -Aus[i, j]
            if(ite>nx):
                M_Au[ite, ite-nx-1] = -Aun[i, j]
            ite+=1
    
    return M_Au, M_Bup, Aup, Aue, Auw, Aun, Aus, Bup
    
    
def V_coeffUpwind(Uxo, Uyo, Po, Uve, Uvw, Vvn, Vvs, D, rho, visc, urf_UV, nx, ny, dx, dy):
    Avp = np.zeros((ny+1, nx))
    Ave = np.zeros((ny+1, nx))
    Avw = np.zeros((ny+1, nx))
    Avn = np.zeros((ny+1, nx))
    Avs = np.zeros((ny+1, nx))
    Bvp = np.zeros((ny+1, nx))
    vyx = visc*dy/dx
    vxy = visc*dx/dy
    for i in range (0, ny+1):
        for j in range (0, nx):
            if(i!=0 and i!=ny):
                Avp[i, j] = rho*dy*(max(Uve[i, j], 0) + max(-Uvw[i, j], 0)) +\
                            rho*dx*(max(Vvn[i, j], 0) + max(-Vvs[i, j], 0)) +\
                            2*(vxy+vyx)
                Ave[i, j] = rho*dy*max(-Uve[i, j], 0) + vyx
                Avw[i, j] = rho*dy*max(Uvw[i, j], 0) + vyx
                Avn[i, j] = rho*dx*max(-Vvn[i, j], 0) + vxy
                Avs[i, j] = rho*dx*max(Vvs[i, j], 0) + vxy
                Bvp[i, j] = dx*(Po[i, j]-Po[i-1, j])
            # West Boundary Outlet
            if(D[0][i, j] == -1 and D[0][i, j+1] == 2):
                Avp[i, j] += -vyx - rho*dy*max(Uvw[i, j], 0)
                Avw[i, j] = 0
            # East Boundary Outlet
            if(D[0][i, j] == 2 and D[0][i, j+1] == -1):
                Avp[i, j] += -vyx - rho*dy*max(-Uve[i, j], 0)
                Ave[i, j] = 0
            # West Boundary Wall
            if(D[0][i, j] == 0 and D[0][i, j+1] == 2):
                Avp[i, j] += 2*vyx
                Ave[i, j] += vyx/3
                Avw[i, j] = 0
                Bvp[i, j] += vyx*8*D[2][i, j]/3
            # East Boundary Wall
            if(D[0][i, j] == 2 and D[0][i, j+1] == 0):
                Avp[i, j] += 2*vyx
                Avw[i, j] += vyx/3
                Ave[i, j] = 0
                Bvp[i, j] += vyx*8*D[2][i, j+1]/3
            # West Boundary Inlet
            if(D[0][i, j] == 1 and D[0][i, j+1] == 2):
                Avp[i, j] += 2*vyx + 2*rho*dy*max(Uvw[i, j], 0)
                Ave[i, j] += vyx/3 + rho*dy*max(Uvw[i, j], 0)/3
                Avw[i, j] = 0
                Bvp[i, j] += vyx*8*D[2][i, j]/3 + 8*rho*dy*D[2][i, j]*max(Uvw[i, j], 0)/3
            # East Boundary Inlet
            if(D[0][i, j] == 2 and D[0][i, j+1] == 1):
                Avp[i, j] += 2*vyx + 2*rho*dy*max(-Uve[i, j], 0)
                Avw[i, j] += vyx/3 + rho*dy*max(-Uve[i, j], 0)/3
                Ave[i, j] = 0
                Bvp[i, j] += vyx*8*D[2][i, j+1]/3 + 8*rho*dy*D[2][i, j+1]*max(-Uve[i, j], 0)/3
            

            # Data Point on Boundary 
            if(D[0][i, j]!=2 and D[0][i, j+1]!=2):
                Avp[i, j] = 1
                Ave[i, j] = 0
                Avw[i, j]  = 0
                Avn[i, j]  = 0
                Avs[i, j]  = 0
                Bvp[i, j]  = Uyo[i, j]
            # Outlet Boundary 
            if(D[0][i, j]==-1 and D[0][i, j+1]==-1 and i==0):
                Avp[i, j] = 1
                Ave[i, j] = 0
                Avw[i, j]  = 0
                Avn[i, j]  = 0
                Avs[i, j]  = 1
                Bvp[i, j]  = 0
            if(D[0][i, j]==-1 and D[0][i, j+1]==-1 and i==ny):
                Avp[i, j] = 1
                Ave[i, j] = 0
                Avw[i, j]  = 0
                Avn[i, j]  = 1
                Avs[i, j]  = 0
                Bvp[i, j]  = 0
            # Under-Relaxation Factor
            if(D[0][i, j]==2 or D[0][i, j+1]==2):
                Avp[i, j] = Avp[i, j]/urf_UV
                Bvp[i, j] += (1-urf_UV)*Avp[i, j]*Uyo[i, j]
    # Matrix Creation
    M_Bvp = Bvp.flatten()
    M_Av = np.zeros(((ny+1)*(nx), (ny+1)*(nx)))
    ite=0
    for i in range (0, ny+1):
        for j in range (0, nx):
            M_Av[ite, ite] = Avp[i, j]
            if ((ite+1)%(nx)!=0):
                M_Av[ite, ite+1] = -Ave[i, j]
            if((ite%(nx)!=0) and (ite!=0)):
                M_Av[ite, ite-1] = -Avw[i, j]
            if (ite<(ny)*(nx)):
                M_Av[ite, ite+nx] = -Avs[i, j]
            if(ite>nx-1):
                M_Av[ite, ite-nx] = -Avn[i, j]
            ite+=1
    
    return M_Av, M_Bvp, Avp, Ave, Avw, Avn, Avs, Bvp


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""

import numpy as np 

def U_face(Uxo, Uyo, D, nx, ny):
    Uue = np.zeros((ny, nx+1))
    Uuw = np.zeros((ny, nx+1))
    Vun = np.zeros((ny, nx+1))
    Vus = np.zeros((ny, nx+1))
    for i in range (0, ny):
        for j in range (0, nx+1):
            if(D[0][i, j]==2 or D[0][i+1, j]==2):
                Uue[i, j] = (Uxo[i, j]+Uxo[i, j+1])/2
                Uuw[i, j] = (Uxo[i, j]+Uxo[i, j-1])/2
                Vun[i, j] = (Uyo[i, j-1]+Uyo[i, j])/2
                Vus[i, j] = (Uyo[i+1, j-1]+Uyo[i+1, j])/2
                
                
    return Uue, Uuw, Vun, Vus
    
    
def V_face(Uxo, Uyo, D, nx, ny):
    Uve = np.zeros((ny+1, nx))
    Uvw = np.zeros((ny+1, nx))
    Vvn = np.zeros((ny+1, nx))
    Vvs = np.zeros((ny+1, nx))
    for i in range (0, ny+1):
        for j in range (0, nx):
            if(D[0][i, j]==2 or D[0][i, j+1]==2):
                Vvs[i, j] = (Uyo[i, j]+Uyo[i+1, j])/2
                Vvn[i, j] = (Uyo[i, j]+Uyo[i-1, j])/2
                Uve[i, j] = (Uxo[i-1, j+1]+Uxo[i, j+1])/2
                Uvw[i, j] = (Uxo[i-1, j]+Uxo[i, j])/2
    return Uve, Uvw, Vvn, Vvs


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""
import numpy as np 

def P_coeffSIMPLE(Aup, Avp, Uxs, Uys, D, nx, ny, dx, dy):
    App =np.zeros((ny, nx))
    Ape =np.zeros((ny, nx))
    Apw =np.zeros((ny, nx))
    Apn =np.zeros((ny, nx))
    Aps =np.zeros((ny, nx))
    Bpp =np.zeros((ny, nx))
    for j in range (0, nx):
        for i in range (0, ny):
            if (D[0][i, j+1]!=2 and D[0][i+1, j+1]!=2):
                Ape[i, j] = 0
            else:
                Ape[i, j] = dy**2/Aup[i, j+1]
            if (D[0][i, j]!=2 and D[0][i+1, j]!=2):
                Apw[i, j] = 0
            else:
                Apw[i, j] = dy**2/Aup[i, j]
            if (D[0][i, j]!=2 and D[0][i, j+1]!=2):
                Apn[i, j] = 0
            else:
                Apn[i, j] = dx**2/Avp[i, j]
            if (D[0][i+1, j]!=2 and D[0][i+1, j+1]!=2):
                Aps[i, j] = 0
            else:
                Aps[i, j] = dx**2/Avp[i+1, j]
            App[i, j] = Ape[i, j] + Apw[i, j] + Apn[i, j] + Aps[i, j]
            Bpp[i, j] = dy*(Uxs[i, j]-Uxs[i, j+1]) + \
                        dx*(Uys[i+1, j]-Uys[i, j])
            if(D[0][i, j]==-1 or D[0][i+1, j]==-1 or D[0][i, j+1]==-1 or D[0][i+1, j+1]==-1):
                App[i, j] = 1
                Ape[i, j] = 0
                Apw[i, j] = 0
                Apn[i, j] = 0
                Aps[i, j] = 0
                Bpp[i, j] = 0
            if(D[0][i, j]!=2 and D[0][i+1, j]!=2 and D[0][i, j+1]!=2 and D[0][i+1, j+1]!=2):
                App[i, j] = 1
                Ape[i, j] = 0
                Apw[i, j] = 0
                Apn[i, j] = 0
                Aps[i, j] = 0
                Bpp[i, j] = 0
                
            
    
    M_Bpp = Bpp.flatten()
    
    M_A = np.zeros((ny*nx, ny*nx))
    ite=0
    for i in range (0, ny):
        for j in range (0, nx):
            M_A[ite, ite] = App[i, j]
            if ((ite+1)%(nx)!=0):
                M_A[ite, ite+1] = -Ape[i, j]
            if(ite%(nx)!=0):
                M_A[ite, ite-1] = -Apw[i, j]
            if (ite<(ny-1)*(nx)):
                M_A[ite, ite+nx] = -Aps[i, j]
            if(ite>nx-1):
                M_A[ite, ite-nx] = -Apn[i, j]
            ite+=1

    
    return M_A, M_Bpp, App, Ape, Apw, Apn, Aps, Bpp


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##########################\/\/\/\/2D_Panel CFD\/\/\/\/##########################
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~______________ ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~ /_____________\   ~   ~   ~   ~   ~   ~   ~  ~  ~  #     
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
# ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  #
#   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~   ~  ~  ~  ~ #
##########################\/\/\/\/\/\/\/\/\/\/\/\/\/\/##########################

"""
LICENSE

    This file is a part of 2D_Panel CFD.
    
    2D_Panel CFD is a repository with 2D framework to test new numerical schemes, 
    pressure coupling algorithms, VOF etc. A GUI is intended to be made shortly 
    to make this a user oriented program.
    Copyright (C) <2021>  <Fluidentity>

    2D_Panel CFD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    2D_Panel CFD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
    
    2D_Panel CFD comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
    
"""

import numpy as np 
from matplotlib import pyplot as plt
import os

def domain(caseFolder):
    # Domain input 
    xMin = float(input("\nEnter X_min value\n"))
    xMax = float(input("\nEnter X_max value\n"))
    dx_temp = float(input("\nEnter Cell size in X-axis [dx]\n"))
    nx = int((xMax-xMin)/dx_temp)
    dx = (xMax-xMin)/nx
    print("\n--- No. of elements in X-axis [{}] ---\n".format(nx))
    yMin = float(input("\nEnter Y_min value\n"))
    yMax = float(input("\nEnter Y_max value\n"))
    dy_temp = float(input("\nEnter Cell size in Y-axis [dy]\n"))
    ny = int((yMax-yMin)/dy_temp)
    dy = (yMax-yMin)/ny
    print("\n--- No. of elements in Y-axis [{}] ---\n".format(ny))
    
    temp = "/boundary.txt"
    boundary_info = open(caseFolder+temp, 'w')
    # Boundary revealing Matrix 
    B1 = 2*np.ones((ny+1, nx+1))
    
    # Scalar Matrix [U, V, P]
    B2 = -1000.5*np.ones((ny+1, nx+1))
    
    # Domain Matrix 
    D = []
    D.append(B1)
    D.append(B2)
    D.append(B2.copy())
    D.append(B2.copy())
    
    # Boundary Definition
    flag_Boundary = 0
    flag_SouthBoundary = 0
    flag_NorthBoundary = 0
    flag_WestBoundary = 0
    flag_EastBoundary = 0
    flag_Outlet = 0
    
    # Boundary Defining Loop
    while (flag_Boundary < 1):
        
        # Southern Boundary Defining Loop
        
        while (flag_SouthBoundary < 1):
            
            flag_Inlet_SouthBoundary = 0
            flag_Outlet_SouthBoundary = 0
            flag_SlidingWall_SouthBoundary = 0
            flag = input("\nInlet in South Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Inlet_SouthBoundary = 1
            flag = input("\nSliding Wall in South Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_SlidingWall_SouthBoundary = 1
            flag = input("\nOutlet in South Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Outlet_SouthBoundary = 1         
            
            # Initialising Boundary Markers
            south_Boundary = np.zeros((3, 6))
            south_Boundary[1, 0] = 1
            south_Boundary[2, 0] = -1
            # Temporary Wall
            south_Boundary[0, 1] = xMin
            south_Boundary[0, 2] = xMax
            D[0][-1, :] = 0
            D[1][-1, :] = 0
            D[2][-1, :] = 0
            # South Boundary Sliding U-velocity
            if(flag_SlidingWall_SouthBoundary==1):
                south_Boundary[0, 3] = float(input("\nEnter U-Velocity for Sliding Wall in South Boundary\n"))
                D[1][-1, 1:nx] = south_Boundary[0, 3]
            # Inlet definition in South Boundary
            if(flag_Inlet_SouthBoundary==1):
                south_Boundary[1, 1] = float(input("\nEnter xmin for Inlet in South Boundary\n"))
                southInlet_min_index = int((south_Boundary[1, 1]-xMin)/dx)
                if(southInlet_min_index==0):
                    southInlet_min_index+=1
                south_Boundary[1, 2] = float(input("\nEnter xmax for Inlet in South Boundary\n"))
                southInlet_max_index = int((south_Boundary[1, 2]-xMin)/dx)
                if(southInlet_max_index==nx):
                    southInlet_max_index-=1
                south_Boundary[1, 4] = float(input("\nEnter V-Velocity for Inlet in South Boundary\n"))
                # Boundary Marker
                D[0][-1, southInlet_min_index:southInlet_max_index+1] = 1.0
                # U-vel - Initialised to 0
                D[1][-1, southInlet_min_index:southInlet_max_index+1] = 0
                # V-vel - User Initialised
                D[2][-1, southInlet_min_index:southInlet_max_index+1] = south_Boundary[1, 4]
                
            # Outlet definition in South Boundary
            if(flag_Outlet_SouthBoundary==1):
                flag_Outlet = 1
                south_Boundary[2, 1] = float(input("\nEnter xmin for Outlet in South Boundary\n"))
                southOutlet_min_index = int((south_Boundary[2, 1]-xMin)/dx)
                if(southOutlet_min_index==0):
                    southOutlet_min_index+=1
                south_Boundary[2, 2] = float(input("\nEnter xmax for Outlet in South Boundary\n"))
                southOutlet_max_index = int((south_Boundary[2, 2]-xMin)/dx)
                if(southOutlet_max_index==nx):
                    southOutlet_max_index-=1
                south_Boundary[2, 5] = float(input("\nEnter Pressure for Outlet in South Boundary\n"))
                # Boundary Marker
                D[0][-1, southOutlet_min_index:southOutlet_max_index+1] = -1.0
                # U-vel - Uninitialised
                D[1][-1, southOutlet_min_index:southOutlet_max_index+1] = -1000.5
                # V-vel - Uninitialised
                D[2][-1, southOutlet_min_index:southOutlet_max_index+1] = -1000.5
                # Pressure - User Initialised
                D[3][-1, southOutlet_min_index:southOutlet_max_index+1] = south_Boundary[2, 5]
            
            
            print("\n\n------------------SOUTH BOUNDARY------------------\n")
            boundary_info.write("\n\n------------------SOUTH BOUNDARY------------------\n")
            ite=0
            while ite<nx+1:
                if(D[0][-1, ite]==0):
                    ite1=ite+1
                    if(ite1==nx+1 and D[0][-1, ite-1]!=0):
                        print("\n[Wall]\n")
                        print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                        print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                        print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                        boundary_info.write("\n[Wall]\n")
                        boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                        boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                        boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                    while ite1<nx+1:
                        if(D[0][-1, ite1]!=0):
                            print("\n[Wall]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            if(flag_SlidingWall_SouthBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[1][-1, ite1-1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[1][-1, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Wall]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            if(flag_SlidingWall_SouthBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[1][-1, ite1-1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[1][-1, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                if(D[0][-1, ite]==1):
                    ite1 = ite+1
                    while ite1<nx+1:
                        if(D[0][-1, ite1]!=1):
                            print("\n[Inlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            print("\nInlet V-Velocity {}\n".format(D[2][-1, ite1-1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\nInlet V-Velocity {}\n".format(D[2][-1, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Inlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            print("\nInlet V-Velocity {}\n".format(D[2][-1, ite1-1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\nInlet V-Velocity {}\n".format(D[2][-1, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                if(D[0][-1, ite]==-1):
                    ite1=ite+1
                    while ite1<nx+1:
                        if(D[0][-1, ite1]!=-1):
                            print("\n[Outlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            print("\nOutlet Pressure {}\n".format(D[3][-1, ite1-1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][-1, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Outlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            print("\nOutlet Pressure {}\n".format(D[3][-1, ite1-1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][-1, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                ite=ite+1
                            
            flag = input("\nFinished with South Boundary Condition ?? [y/n]\n")
            if(flag=='y'):
                flag_SouthBoundary = 1
                print("\n--------------------------------------------------\n")
                boundary_info.write("\n--------------------------------------------------\n")
        
        # Northern Boundary Defining Loop    
        
        while (flag_NorthBoundary < 1):
            
            flag_Inlet_NorthBoundary = 0
            flag_Outlet_NorthBoundary = 0
            flag_SlidingWall_NorthBoundary = 0
            flag = input("\nInlet in North Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Inlet_NorthBoundary = 1
            flag = input("\nSliding Wall in North Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_SlidingWall_NorthBoundary = 1
            flag = input("\nOutlet in North Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Outlet_NorthBoundary = 1         
            
            # Initialising Boundary Markers
            north_Boundary = np.zeros((3, 6))
            north_Boundary[1, 0] = 1
            north_Boundary[2, 0] = -1
            # Temporary Wall
            north_Boundary[0, 1] = xMin
            north_Boundary[0, 2] = xMax
            D[0][0, :] = 0
            D[1][0, :] = 0
            D[2][0, :] = 0
            # North Boundary Sliding U-velocity
            if(flag_SlidingWall_NorthBoundary==1):
                north_Boundary[0, 3] = float(input("\nEnter U-Velocity for Sliding Wall in North Boundary\n"))
                D[1][0, 1:nx] = north_Boundary[0, 3]
            # Inlet definition in North Boundary
            if(flag_Inlet_NorthBoundary==1):
                north_Boundary[1, 1] = float(input("\nEnter xmin for Inlet in North Boundary\n"))
                northInlet_min_index = int((north_Boundary[1, 1]-xMin)/dx)
                if(northInlet_min_index==0):
                    northInlet_min_index+=1
                north_Boundary[1, 2] = float(input("\nEnter xmax for Inlet in North Boundary\n"))
                northInlet_max_index = int((north_Boundary[1, 2]-xMin)/dx)
                if(northInlet_max_index==nx):
                    northInlet_max_index-=1
                north_Boundary[1, 4] = float(input("\nEnter V-Velocity for Inlet in North Boundary\n"))
                # Boundary Marker
                D[0][0, northInlet_min_index:northInlet_max_index+1] = 1.0
                # U-vel Initialised to 0
                D[1][0, northInlet_min_index:northInlet_max_index+1] = 0.0
                # V-vel User Initialised
                D[2][0, northInlet_min_index:northInlet_max_index+1] = north_Boundary[1, 4]
            # Outlet definition in North Boundary
            if(flag_Outlet_NorthBoundary==1):
                flag_Outlet = 1
                north_Boundary[2, 1] = float(input("\nEnter xmin for Outlet in North Boundary\n"))
                northOutlet_min_index = int((north_Boundary[2, 1]-xMin)/dx)
                if(northOutlet_min_index==0):
                    northOutlet_min_index+=1
                north_Boundary[2, 2] = float(input("\nEnter xmax for Outlet in North Boundary\n"))
                northOutlet_max_index = int((north_Boundary[2, 2]-xMin)/dx)
                if(northOutlet_max_index==nx):
                    northOutlet_max_index-=1
                north_Boundary[2, 5] = float(input("\nEnter Pressure for Outlet in North Boundary\n"))
                # Boundary Marker
                D[0][0, northOutlet_min_index:northOutlet_max_index+1] = -1.0
                # U-vel Uninitialised
                D[1][0, northOutlet_min_index:northOutlet_max_index+1] = -1000.5
                # V-vel Uninitialised
                D[2][0, northOutlet_min_index:northOutlet_max_index+1] = -1000.5
                # Pressure User Initialised
                D[3][0, northOutlet_min_index:northOutlet_max_index+1] = north_Boundary[2, 5]
            
            
            print("\n\n------------------NORTH BOUNDARY------------------\n")
            boundary_info.write("\n\n------------------NORTH BOUNDARY------------------\n")
            ite=0
            while ite<nx+1:
                if(D[0][0, ite]==0):
                    ite1=ite+1
                    if(ite1==nx+1 and D[0][0, ite-1]!=0):
                        print("\n[Wall]\n")
                        print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                        print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                        print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                        boundary_info.write("\n[Wall]\n")
                        boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                        boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                        boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                    while ite1<nx+1:
                        if(D[0][0, ite1]!=0):
                            print("\n[Wall]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            if(flag_SlidingWall_NorthBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[1][0, ite1-1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[1][0, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Wall]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            if(flag_SlidingWall_NorthBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[1][0, ite1-1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[1][0, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                if(D[0][0, ite]==1):
                    ite1 = ite+1
                    while ite1<nx+1:
                        if(D[0][0, ite1]!=1):
                            print("\n[Inlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            print("\nInlet V-Velocity {}\n".format(D[2][0, ite1-1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\nInlet V-Velocity {}\n".format(D[2][0, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Inlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            print("\nInlet V-Velocity {}\n".format(D[2][0, ite1-1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\nInlet V-Velocity {}\n".format(D[2][0, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                if(D[0][0, ite]==-1):
                    ite1=ite+1
                    while ite1<nx+1:
                        if(D[0][0, ite1]!=-1):
                            print("\n[Outlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            print("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            print("\nOutlet Pressure {}\n".format(D[3][0, ite1-1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1-1)*dx, ite1-1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-1-ite+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][0, ite1-1]))
                            ite=ite1-1
                            break
                        if(ite1==nx):
                            print("\n[Outlet]\n")
                            print("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            print("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            print("\nOutlet Pressure {}\n".format(D[3][0, ite1-1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nxMin = [{}] Starting Index = [{}]\n ".format(xMin+ite*dx, ite))
                            boundary_info.write("\nxMax = [{}] Ending Index = [{}]\n ".format(xMin+(ite1)*dx, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite1-ite+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][0, ite1-1]))
                            ite=ite1-1
                            break
                        ite1=ite1+1
                ite=ite+1
                            
            flag = input("\nFinished with North Boundary Condition ?? [y/n]\n")
            if(flag=='y'):
                flag_NorthBoundary = 1
                print("\n--------------------------------------------------\n")
                boundary_info.write("\n--------------------------------------------------\n")
                
        # Western Boundary Defining Loop
        
        while (flag_WestBoundary < 1):
            
            flag_Inlet_WestBoundary = 0
            flag_Outlet_WestBoundary = 0
            flag_SlidingWall_WestBoundary = 0
            flag = input("\nInlet in West Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Inlet_WestBoundary = 1
            flag = input("\nSliding Wall in West Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_SlidingWall_WestBoundary = 1
            flag = input("\nOutlet in West Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Outlet_WestBoundary = 1         
            
            # Initialising Boundary Markers
            west_Boundary = np.zeros((3, 6))
            west_Boundary[1, 0] = 1
            west_Boundary[2, 0] = -1
            # Temporary Wall
            west_Boundary[0, 1] = yMin
            west_Boundary[0, 2] = yMax
            D[0][:, 0] = 0
            D[1][:, 0] = 0
            D[2][:, 0] = 0
            # West Boundary Sliding U-velocity
            if(flag_SlidingWall_WestBoundary==1):
                west_Boundary[0, 4] = float(input("\nEnter V-Velocity for Sliding Wall in West Boundary\n"))
                D[2][1:ny, 0] = west_Boundary[0, 4]
            # Inlet definition in West Boundary
            if(flag_Inlet_WestBoundary==1):
                west_Boundary[1, 1] = float(input("\nEnter ymin for Inlet in West Boundary\n"))
                westInlet_bottom_index = int((yMax-west_Boundary[1, 1])/dy)
                if(westInlet_bottom_index==ny):
                    westInlet_bottom_index-=1
                west_Boundary[1, 2] = float(input("\nEnter ymax for Inlet in West Boundary\n"))
                westInlet_top_index = int((yMax-west_Boundary[1, 2])/dy)
                if(westInlet_top_index==0):
                    westInlet_top_index+=1
                west_Boundary[1, 3] = float(input("\nEnter U-Velocity for Inlet in West Boundary\n"))
                # Boundary Marker
                D[0][westInlet_top_index:westInlet_bottom_index+1, 0] = 1.0
                # U-vel User Initialised
                D[1][westInlet_top_index:westInlet_bottom_index+1, 0] = west_Boundary[1, 3]
                # V-vel Initialised to 0
                D[2][westInlet_top_index:westInlet_bottom_index+1, 0] = 0.0
            # Outlet definition in West Boundary
            if(flag_Outlet_WestBoundary==1):
                flag_Outlet = 1
                west_Boundary[2, 1] = float(input("\nEnter ymin for Outlet in West Boundary\n"))
                westOutlet_bottom_index = int((yMax-west_Boundary[2, 1])/dy)
                if(westOutlet_bottom_index==ny):
                    westOutlet_bottom_index-=1
                west_Boundary[2, 2] = float(input("\nEnter ymax for Outlet in West Boundary\n"))
                westOutlet_top_index = int((yMax-west_Boundary[2, 2])/dy)
                if(westOutlet_top_index==0):
                    westOutlet_top_index+=1
                west_Boundary[2, 5] = float(input("\nEnter Pressure for Outlet in West Boundary\n"))
                # Boundary Marker
                D[0][westOutlet_top_index:westOutlet_bottom_index+1, 0] = -1.0
                # U-vel Uninitialised
                D[1][westOutlet_top_index:westOutlet_bottom_index+1, 0] = -1000.5
                # V-vel Uninitialised
                D[2][westOutlet_top_index:westOutlet_bottom_index+1, 0] = -1000.5
                # Pressure User Initialised
                D[3][westOutlet_top_index:westOutlet_bottom_index+1, 0] = west_Boundary[2, 5]
            
            
            print("\n\n------------------WEST BOUNDARY-----------------\n")
            boundary_info.write("\n\n------------------WEST BOUNDARY-----------------\n")
            ite=ny
            while ite>-1:
                if(D[0][ite, 0]==0):
                    ite1=ite-1
                    if(ite1==-1 and D[0][ite+1, 0]!=0):
                        print("\n[Wall]\n")
                        print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                        print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                        print("\nNo. of elements = [{}] \n".format(ite-ite1))
                        boundary_info.write("\n[Wall]\n")
                        boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                        boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                        boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                    while ite1>-1:
                        if(D[0][ite1, 0]!=0):
                            print("\n[Wall]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                            if(flag_SlidingWall_WestBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, 0]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, 0]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Wall]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            if(flag_SlidingWall_WestBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, 0]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, 0]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                if(D[0][ite, 0]==1):
                    ite1 = ite-1
                    while ite1>-1:
                        if(D[0][ite1, 0]!=1):
                            print("\n[Inlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1))
                            print("\nInlet U-Velocity {}\n".format(D[1][ite1+1, 0]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                            boundary_info.write("\nInlet U-Velocity {}\n".format(D[1][ite1+1, 0]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Inlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            print("\nInlet U-Velocity {}\n".format(D[1][ite1+1, 0]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            boundary_info.write("\nInlet U-Velocity {}\n".format(D[1][ite1+1, 0]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                if(D[0][ite, 0]==-1):
                    ite1=ite-1
                    while ite1>-1:
                        if(D[0][ite1, 0]!=-1):
                            print("\n[Outlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1))
                            print("\nOutlet Pressure {}\n".format(D[3][ite1+1, 0]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][ite1+1, 0]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Outlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            print("\nOutlet Pressure {}\n".format(D[3][ite1+1, 0]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][ite1+1, 0]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                ite=ite-1
                            
            flag = input("\nFinished with West Boundary Condition ?? [y/n]\n")
            if(flag=='y'):
                flag_WestBoundary = 1
                print("\n-------------------------------------------------\n")
                boundary_info.write("\n-------------------------------------------------\n")
        
        # Eastern Boundary Defining Loop
        
        while (flag_EastBoundary < 1):
            
            flag_Inlet_EastBoundary = 0
            flag_Outlet_EastBoundary = 0
            flag_SlidingWall_EastBoundary = 0
            flag = input("\nInlet in East Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Inlet_EastBoundary = 1
            flag = input("\nSliding Wall in East Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_SlidingWall_EastBoundary = 1
            flag = input("\nOutlet in East Boundary ?? [y/n]\n")
            if(flag=='y'):
                flag_Outlet_EastBoundary = 1         
            
            # Initialising Boundary Markers
            east_Boundary = np.zeros((3, 6))
            east_Boundary[1, 0] = 1
            east_Boundary[2, 0] = -1
            # Temporary Wall
            east_Boundary[0, 1] = yMin
            east_Boundary[0, 2] = yMax
            D[0][:, -1] = 0
            D[1][:, -1] = 0
            D[2][:, -1] = 0
            # East Boundary Sliding U-velocity
            if(flag_SlidingWall_EastBoundary==1):
                east_Boundary[0, 4] = float(input("Enter V-Velocity for Sliding Wall in East Boundary\n"))
                D[2][1:ny, -1] = east_Boundary[0, 4]
            # Inlet definition in East Boundary
            if(flag_Inlet_EastBoundary==1):
                east_Boundary[1, 1] = float(input("Enter ymin for Inlet in East Boundary\n"))
                eastInlet_bottom_index = int((yMax-east_Boundary[1, 1])/dy)
                if(eastInlet_bottom_index==ny):
                    eastInlet_bottom_index-=1
                east_Boundary[1, 2] = float(input("Enter ymax for Inlet in East Boundary\n"))
                eastInlet_top_index = int((yMax-east_Boundary[1, 2])/dy)
                if(eastInlet_top_index==0):
                    eastInlet_top_index+=1
                east_Boundary[1, 3] = float(input("Enter U-Velocity for Inlet in East Boundary\n"))
                # Boundary Marker
                D[0][eastInlet_top_index:eastInlet_bottom_index+1, -1] = 1.0
                # U-vel User Initialised
                D[1][eastInlet_top_index:eastInlet_bottom_index+1, -1] = east_Boundary[1, 3]
                # V-vel Initialised to 0
                D[2][eastInlet_top_index:eastInlet_bottom_index+1, -1] = 0.0
            # Outlet definition in East Boundary
            if(flag_Outlet_EastBoundary==1):
                flag_Outlet = 1
                east_Boundary[2, 1] = float(input("Enter ymin for Outlet in East Boundary\n"))
                eastOutlet_bottom_index = int((yMax-east_Boundary[2, 1])/dy)
                if(eastOutlet_bottom_index==ny):
                    eastOutlet_bottom_index-=1
                east_Boundary[2, 2] = float(input("Enter ymax for Outlet in East Boundary\n"))
                eastOutlet_top_index = int((yMax-east_Boundary[2, 2])/dy)
                if(eastOutlet_top_index==0):
                    eastOutlet_top_index+=1
                east_Boundary[2, 5] = float(input("Enter Pressure for Outlet in East Boundary\n"))
                # Boundary Marker
                D[0][eastOutlet_top_index:eastOutlet_bottom_index+1, -1] = -1.0
                # U-vel Uninitialised
                D[1][eastOutlet_top_index:eastOutlet_bottom_index+1, -1] = -1000.5
                # V-vel Uninitialised
                D[2][eastOutlet_top_index:eastOutlet_bottom_index+1, -1] = -1000.5
                # Pressure User Initialised
                D[3][eastOutlet_top_index:eastOutlet_bottom_index+1, -1] = east_Boundary[2, 5]
            
            
            print("\n\n------------------EAST BOUNDARY-----------------\n")
            boundary_info.write("\n\n------------------EAST BOUNDARY-----------------\n")
            ite=ny
            while ite>-1:
                if(D[0][ite, -1]==0):
                    ite1=ite-1
                    if(ite1==-1 and D[0][ite+1, -1]!=0):
                        print("\n[Wall]\n")
                        print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                        print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                        print("\nNo. of elements = [{}] \n".format(ite-ite1))
                        boundary_info.write("\n[Wall]\n")
                        boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                        boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                        boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                    while ite1>-1:
                        if(D[0][ite1, -1]!=0):
                            print("\n[Wall]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}]\n".format(ite-ite1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}]\n".format(ite-ite1))
                            if(flag_SlidingWall_EastBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, -1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, -1]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Wall]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}]\n".format(ite-ite1+1))
                            boundary_info.write("\n[Wall]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}]\n".format(ite-ite1+1))
                            if(flag_SlidingWall_EastBoundary==1):
                                print("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, -1]))
                                boundary_info.write("\nSliding Wall Velocity {}\n".format(D[2][ite1+1, -1]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                if(D[0][ite, -1]==1):
                    ite1 = ite-1
                    while ite1>-1:
                        if(D[0][ite1, -1]!=1):
                            print("\n[Inlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1))
                            print("\nInlet U-Velocity {}\n".format(D[1][ite1+1, -1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                            boundary_info.write("\nInlet U-Velocity {}\n".format(D[1][ite1+1, -1]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Inlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            print("\nInlet U-Velocity {}\n".format(D[1][ite1+1, -1]))
                            boundary_info.write("\n[Inlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            boundary_info.write("\nInlet U-Velocity {}\n".format(D[1][ite1+1, -1]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                if(D[0][ite, -1]==-1):
                    ite1=ite-1
                    while ite1>-1:
                        if(D[0][ite1, -1]!=-1):
                            print("\n[Outlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1))
                            print("\nOutlet Pressure {}\n".format(D[3][ite1+1, -1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1+1)*dy, ite1+1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][ite1+1, -1]))
                            ite=ite1+1
                            break
                        if(ite1==0):
                            print("\n[Outlet]\n")
                            print("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            print("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            print("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            print("\nOutlet Pressure {}\n".format(D[3][ite1+1, -1]))
                            boundary_info.write("\n[Outlet]\n")
                            boundary_info.write("\nyMin = [{:.2f}] Bottom Index = [{}]\n ".format(yMax-(ite)*dy, ite))
                            boundary_info.write("\nyMax = [{:.2f}] Top Index = [{}]\n ".format(yMax-(ite1)*dy, ite1))
                            boundary_info.write("\nNo. of elements = [{}] \n".format(ite-ite1+1))
                            boundary_info.write("\nOutlet Pressure {}\n".format(D[3][ite1+1, -1]))
                            ite=ite1+1
                            break
                        ite1=ite1-1
                ite=ite-1
                            
            flag = input("Finished with East Boundary Condition ?? [y/n]")
            if(flag=='y'):
                flag_EastBoundary = 1
                print("\n-------------------------------------------------\n")
                boundary_info.write("\n-------------------------------------------------\n")
        flag_Boundary = 1
        
    return D, xMin, xMax, yMin, yMax, dx, dy, nx, ny, south_Boundary, north_Boundary, west_Boundary, east_Boundary, flag_Outlet

def obstacle(D, xMin, xMax, yMin, yMax, nx, ny, dx, dy):
    package_dir = os.path.dirname(os.path.realpath(__file__))
    flag_obstacle = 'n'
    while flag_obstacle!='y':
        scal_x = float(input("\nEnter Scaling factor for X-direction\n"))
        scal_y = float(input("\nEnter Scaling factor for Y-direction\n"))
        traf_x = float(input("\nOffset in X\n"))
        traf_y = float(input("\nOffset in Y\n")) + dy/10
        rot = float(input("\nEnter anti-clockwise rotation angle\n"))
        flag_geometryCase = input("\n1. Enter address of .txt file\n2. Use default address\n")
        if(flag_geometryCase==1):
            geometryCase = input("\nEnter address of .txt file\nExample: /home/user/folder/Case_1/Points.txt\n")
        else:
            geometryCase = package_dir+"/Points.txt"
        file1 = open(geometryCase, 'r')
        Lines = file1.readlines()
        P=[]
        N=0
        for line in Lines:
            N+=1
            P.append(np.fromstring(line.strip(), dtype=float, sep=' '))
        N+=1
        P.append(np.fromstring(Lines[0].strip(), dtype=float, sep=' '))
        Lx = xMax-xMin
        Ly = yMax-yMin
        Points_x=[]
        Points_y=[]
        for i in range (0, N):
            Points_x.append(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan((P[i][1])/(P[i][0])))+traf_x)
            Points_y.append(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0]))+traf_y)
        fig = plt.subplots(figsize =(Lx*5, Ly*5))
        plt.xlim(xMin, xMax)
        plt.ylim(yMin, yMax)
        plt.fill(Points_x, Points_y, color = 'black')
        plt.show()
        
        flag_obstacle = (input("\nConitnue [Y/N] ??\n"))
        if(flag_obstacle!='y'):
            continue
        for i in range(0, ny+1):
            for j in range (0, nx+1):
                X = xMin + j*dx
                Y = yMax - i*dy
                count=0
                for ite in range (0, N-1):
                    X1 = Points_x[ite]
                    X2 = Points_x[ite+1]
                    Y1 = Points_y[ite]
                    Y2 = Points_y[ite+1]
                    if(Y1<=Y<=Y2 or Y2<=Y<=Y1):
                        if(X1<=X2):
                            if(X<=(X2+(Y-Y2)*(X1-X2)/(Y1-Y2))):
                                count +=1
                        if(X2<=X1):
                            if(X<=(X1+(Y-Y1)*(X2-X1)/(Y2-Y1))):
                                count +=1
                if(count==1):
                    D[0][i, j] = 0
                    D[1][i, j] = 0
                    D[2][i, j] = 0
                    D[3][i, j] = -100000
    
    return D, P, Points_x, Points_y

def obstacleMath(D, xMin, xMax, yMin, yMax, nx, ny, dx, dy):
    flag_obstacle = 'n'
    while flag_obstacle!='y':
        scal_x = float(input("\nEnter Scaling factor for X-direction\n"))
        scal_y = float(input("\nEnter Scaling factor for Y-direction\n"))
        traf_x = float(input("\nOffset in X\n"))
        traf_y = float(input("\nOffset in Y\n")) + dy/10
        rot = int(input("\nEnter anti-clockwise rotation angle\n"))
            
        P=[]
        asq = float(input("\nEnter Horizontal axis length for Elliptical Geometry\n"))
        bsq = float(input("\nEnter Vertical axis length for Elliptical Geometry\n"))
        a=asq**2
        b=bsq**2
        N=0
        for theta in range (0, 360):
            if(theta%2==0):
                temp1=a*b/(b+a*np.tan(theta*np.pi/180)**2)
                if(0<=theta<=90):
                    temp = [np.abs(np.sqrt(temp1)), np.abs(np.tan(theta*np.pi/180))*(np.abs(np.sqrt(temp1)))]
                elif(90<=theta<=180):
                    temp = [-np.abs(np.sqrt(temp1)), np.abs(np.tan(theta*np.pi/180))*(np.abs(np.sqrt(temp1)))]
                elif(180<=theta<=270):
                    temp = [-np.abs(np.sqrt(temp1)), -np.abs(np.tan(theta*np.pi/180))*(np.abs(np.sqrt(temp1)))]
                elif(270<=theta<=360):
                    temp = [np.abs(np.sqrt(temp1)), -np.abs(np.tan(theta*np.pi/180))*(np.abs(np.sqrt(temp1)))]
                P.append(temp)
                N+=1
        P.append(P[0])
        Lx = xMax-xMin
        Ly = yMax-yMin
        Points_x=[]
        Points_y=[]
        for i in range(0, 181):
            if(0<=(i+rot/2)<=45):
                Points_x.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(45<=(i+rot/2)<=90):
                Points_x.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(90<=(i+rot/2)<=135):
                Points_x.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(135<=(i+rot/2)<=180):
                Points_x.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(180<=(i+rot/2)<=225):
                Points_x.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(-45<=(i+rot/2)<=0):
                Points_x.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(-90<=(i+rot/2)<=-45):
                Points_x.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(-135<=(i+rot/2)<=-90):
                Points_x.append(-np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
            elif(-180<=(i+rot/2)<=-135):
                Points_x.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.cos(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_x)
                Points_y.append(np.abs(np.sqrt((P[i][0]*scal_x)**2 + (P[i][1]*scal_y)**2)*np.sin(rot*np.pi/180+np.arctan(P[i][1]/P[i][0])))+traf_y)
        fig = plt.subplots(figsize =(Lx*5, Ly*5))
        plt.xlim(xMin, xMax)
        plt.ylim(yMin, yMax)
        plt.fill(Points_x, Points_y, color='black')
        plt.show()
        
        flag_obstacle = (input("\nConitnue [Y/N] ??\n"))
        if(flag_obstacle!='y'):
            continue
        for i in range(0, ny+1):
            for j in range (0, nx+1):
                X = xMin + j*dx
                Y = yMax - i*dy
                count=0
                for ite in range (0, 180):
                    X1 = Points_x[ite]
                    X2 = Points_x[ite+1]
                    Y1 = Points_y[ite]
                    Y2 = Points_y[ite+1]
                    if(Y1<Y<=Y2 or Y2<=Y<Y1):
                        if(X<=X1 or X<=X2):
                            count +=1
                if(count==1):
                    D[0][i, j] = 0
                    D[1][i, j] = 0
                    D[2][i, j] = 0
                    D[3][i, j] = -100000
        

    return D, P, Points_x, Points_y